/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizactivate.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizactivate.entity.BizActivate;
import vip.xiaonuo.biz.modular.bizactivate.mapper.BizActivateMapper;
import vip.xiaonuo.biz.modular.bizactivate.param.BizActivateAddParam;
import vip.xiaonuo.biz.modular.bizactivate.param.BizActivateEditParam;
import vip.xiaonuo.biz.modular.bizactivate.param.BizActivateIdParam;
import vip.xiaonuo.biz.modular.bizactivate.param.BizActivatePageParam;
import vip.xiaonuo.biz.modular.bizactivate.service.BizActivateService;

import java.util.List;

/**
 * 用户充值待激活记录Service接口实现类
 *
 * @author fanzherong
 * @date  2025/02/27 16:36
 **/
@Service
public class BizActivateServiceImpl extends ServiceImpl<BizActivateMapper, BizActivate> implements BizActivateService {

    @Override
    public Page<BizActivate> page(BizActivatePageParam bizActivatePageParam) {
        QueryWrapper<BizActivate> queryWrapper = new QueryWrapper<BizActivate>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizActivatePageParam.getUserName())){
            queryWrapper.like("su.name",bizActivatePageParam.getUserName());
        }
        if(ObjectUtil.isNotEmpty(bizActivatePageParam.getStatus())){
            queryWrapper.eq("ba.status",bizActivatePageParam.getStatus());
        }
        queryWrapper.orderByDesc("ba.create_time");
        return this.getBaseMapper().getPageList(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizActivateAddParam bizActivateAddParam) {
        BizActivate bizActivate = BeanUtil.toBean(bizActivateAddParam, BizActivate.class);
        this.save(bizActivate);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizActivateEditParam bizActivateEditParam) {
        BizActivate bizActivate = this.queryEntity(bizActivateEditParam.getId());
        BeanUtil.copyProperties(bizActivateEditParam, bizActivate);
        this.updateById(bizActivate);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizActivateIdParam> bizActivateIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizActivateIdParamList, BizActivateIdParam::getId));
    }

    @Override
    public BizActivate detail(BizActivateIdParam bizActivateIdParam) {
        return this.queryEntity(bizActivateIdParam.getId());
    }

    @Override
    public BizActivate queryEntity(String id) {
        BizActivate bizActivate = this.getById(id);
        if(ObjectUtil.isEmpty(bizActivate)) {
            throw new CommonException("用户充值待激活记录不存在，id值为：{}", id);
        }
        return bizActivate;
    }
}
